@@ -12,7 +12,6 @@ require_once 'PEAR.php';
 /** Import generic tsv data.               */ define('IMPORT_TSV', 7);
 /** Import Mulberry address book data      */ define('IMPORT_MULBERRY', 8);
 /** Import Pine address book data.         */ define('IMPORT_PINE', 9);
-/** Import palm datebook (.pdb).           */ define('IMPORT_PDB', 10);
 /** Import file.                           */ define('IMPORT_FILE', 11);
 /** Import data.                           */ define('IMPORT_DATA', 12);
 
@@ -27,9 +26,9 @@ require_once 'PEAR.php';
  * Abstract class to handle different kinds of Data formats and to
  * help data exchange between Horde applications and external sources.
  *
- * $Horde: framework/Data/Data.php,v 1.80.10.5 2005/07/03 04:39:56 selsky Exp $
+ * $Horde: framework/Data/Data.php,v 1.80.10.14 2007/01/02 13:54:12 jan Exp $
  *
- * Copyright 1999-2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2007 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -41,9 +40,28 @@ require_once 'PEAR.php';
  */
 class Horde_Data extends PEAR {
 
+    /**
+     * File extension.
+     *
+     * @var string
+     */
     var $_extension;
+
+    /**
+     * MIME content type.
+     *
+     * @var string
+     */
     var $_contentType = 'text/plain';
 
+    /**
+     * A list of warnings raised during the last operation.
+     *
+     * @var array
+     * @since Horde 3.1
+     */
+    var $_warnings = array();
+
     /**
      * Stub to import passed data.
      */
@@ -63,9 +81,7 @@ class Horde_Data extends PEAR {
      */
     function importFile($filename, $header = false)
     {
-        $fp = @fopen($filename, 'r');
-        $data = @fread($fp, @filesize($filename));
-        @fclose($fp);
+        $data = file_get_contents($filename);
         return $this->importData($data, $header);
     }
 
@@ -111,6 +127,19 @@ class Horde_Data extends PEAR {
         return $this->_contentType;
     }
 
+    /**
+     * Returns a list of warnings that have been raised during the last
+     * operation.
+     *
+     * @since Horde 3.1
+     *
+     * @return array  A (possibly empty) list of warnings.
+     */
+    function warnings()
+    {
+        return $this->_warnings;
+    }
+
     /**
      * Attempts to return a concrete Horde_Data instance based on $format.
      *
@@ -132,7 +161,8 @@ class Horde_Data extends PEAR {
         $format = basename($format);
 
         if (empty($format) || (strcmp($format, 'none') == 0)) {
-            return $ret = &new Horde_Data();
+            $data =& new Horde_Data();
+            return $data;
         }
 
         if (!empty($app)) {
@@ -142,10 +172,12 @@ class Horde_Data extends PEAR {
         }
         $class = 'Horde_Data_' . $format;
         if (class_exists($class)) {
-            return $ret = &new $class();
+            $data =& new $class();
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $data = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $data;
     }
 
     /**
@@ -182,36 +214,44 @@ class Horde_Data extends PEAR {
     /**
      * Maps a date/time string to an associative array.
      *
+     * The method signature has changed in Horde 3.1.3.
+     *
      * @access private
      *
-     * @param string $date       The date.
-     * @param string $type       One of 'date', 'time' or 'datetime'.
-     * @param string $delimiter  The character that seperates the different
-     *                           date/time parts.
-     * @param string $format     If 'ampm' and $date contains a time we assume
-     *                           that it is in AM/PM format.
+     * @param string $date   The date.
+     * @param string $type   One of 'date', 'time' or 'datetime'.
+     * @param array $params  Two-dimensional array with additional information
+     *                       about the formatting. Possible keys are:<pre>
+     *                       delimiter -- The character that seperates the
+     *                                    different date/time parts.
+     *                       format -- If 'ampm' and $date contains a time we
+     *                                 assume that it is in AM/PM format.
+     *                       order -- If $type is 'datetime' the order of the
+     *                                day and time parts: -1 (timestamp), 0
+     *                                (day/time), 1 (time/day).
+     * @param integer $key   The key to use for $params.
      *
      * @return string  The date or time in ISO format.
      */
-    function mapDate($date, $type, $delimiter, $format = null)
+    function mapDate($date, $type, $params, $key)
     {
         switch ($type) {
         case 'date':
         case 'monthday':
         case 'monthdayyear':
-            $dates = explode($delimiter, $date);
+            $dates = explode($params['delimiter'][$key], $date);
             if (count($dates) != 3) {
                 return $date;
             }
-            $index = array_flip(explode('/', $format));
+            $index = array_flip(explode('/', $params['format'][$key]));
             return $dates[$index['year']] . '-' . $dates[$index['month']] . '-' . $dates[$index['mday']];
 
         case 'time':
-            $dates = explode($delimiter, $date);
+            $dates = explode($params['delimiter'][$key], $date);
             if (count($dates) < 2 || count($dates) > 3) {
                 return $date;
             }
-            if ($format == 'ampm') {
+            if ($params['format'][$key] == 'ampm') {
                 if (strpos(strtolower($dates[count($dates)-1]), 'pm') !== false) {
                     if ($dates[0] !== '12') {
                         $dates[0] += 12;
@@ -219,12 +259,32 @@ class Horde_Data extends PEAR {
                 } elseif ($dates[0] == '12') {
                     $dates[0] = '0';
                 }
-                $dates[count($dates)-1] = (int) $dates[count($dates)-1];
+                $dates[count($dates) - 1] = sprintf('%02d', $dates[count($dates)-1]);
             }
             return $dates[0] . ':' . $dates[1] . (count($dates) == 3 ? (':' . $dates[2]) : ':00');
 
         case 'datetime':
-            return '';
+            switch ($params['order'][$key]) {
+            case -1:
+                return (string)(int)$date == $date
+                    ? date('Y-m-d H:i:s', $date)
+                    : $date;
+            case 0:
+                list($day, $time) = explode(' ', $date, 2);
+                break;
+            case 1:
+               list($time, $day) = explode(' ', $date, 2);
+               break;
+            }
+            $date = $this->mapDate($day, 'date',
+                                   array('delimiter' => $params['day_delimiter'],
+                                         'format' => $params['day_format']),
+                                   $key);
+            $time = $this->mapDate($time, 'time',
+                                   array('delimiter' => $params['time_delimiter'],
+                                         'format' => $params['time_format']),
+                                   $key);
+            return $date . ' ' . $time;
 
         }
     }
@@ -300,8 +360,13 @@ class Horde_Data extends PEAR {
         case IMPORT_DATETIME:
         case IMPORT_DATA:
             if ($action == IMPORT_DATETIME) {
-                $delimiter = Util::getFormData('delimiter');
-                $format = Util::getFormData('format');
+                $params = array('delimiter' => Util::getFormData('delimiter'),
+                                'format' => Util::getFormData('format'),
+                                'order' => Util::getFormData('order'),
+                                'day_delimiter' => Util::getFormData('day_delimiter'),
+                                'day_format' => Util::getFormData('day_format'),
+                                'time_delimiter' => Util::getFormData('time_delimiter'),
+                                'time_format' => Util::getFormData('time_format'));
             }
             if (!isset($_SESSION['import_data']['data'])) {
                 return PEAR::raiseError(_("The uploaded data was lost since the previous step."));
@@ -317,7 +382,7 @@ class Horde_Data extends PEAR {
                             !empty($val) &&
                             isset($param['time_fields']) &&
                             isset($param['time_fields'][$mapped_key])) {
-                            $val = $this->mapDate($val, $param['time_fields'][$mapped_key], $delimiter[$key], $format[$key]);
+                            $val = $this->mapDate($val, $param['time_fields'][$mapped_key], $params, $key);
                         }
                         $data_row[$_SESSION['import_data']['map'][$key]] = $val;
                     }
