@@ -16,24 +16,33 @@ define('DATATREE_FORMAT_FLAT', 2);
 define('DATATREE_ROOT', -1);
 
 /**
- * The DataTree:: class provides a common abstracted interface into
- * the various backends for the Horde DataTree system.
+ * Build a normal select query.
+ */
+define('DATATREE_BUILD_SELECT', 0);
+
+/**
+ * Build a count only query.
+ */
+define('DATATREE_BUILD_COUNT', 1);
+
+/**
+ * The DataTree:: class provides a common abstracted interface into the
+ * various backends for the Horde DataTree system.
  *
- * A piece of data is just a title that is saved in the page for the
- * null driver or can be saved in a database to be accessed from
- * everywhere. Every stored object must have a different name (inside
- * each groupid).
+ * A piece of data is just a title that is saved in the page for the null
+ * driver or can be saved in a database to be accessed from everywhere. Every
+ * stored object must have a different name (inside each groupid).
  *
  * Required values for $params:<pre>
- *   'group'         Define each group of objects we want to build.</pre>
+ *   'group' -- Define each group of objects we want to build.</pre>
  *
- * $Horde: framework/DataTree/DataTree.php,v 1.151.2.13 2005/09/21 13:02:08 jan Exp $
+ * $Horde: framework/DataTree/DataTree.php,v 1.151.2.19 2007/01/02 13:54:13 jan Exp $
  *
- * Copyright 1999-2003 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
  *
- * See the enclosed file COPYING for license information (LGPL). If you
- * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you did
+ * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Stephane Huther <shuther@bigfoot.com>
  * @author  Chuck Hagenbuch <chuck@horde.org>
@@ -58,6 +67,13 @@ class DataTree {
      */
     var $_nameMap = array();
 
+     /**
+     * Actual attribute sorting hash.
+     *
+     * @var array
+     */
+    var $_sortHash = null;
+
     /**
      * Hash containing connection parameters.
      *
@@ -124,6 +140,16 @@ class DataTree {
         return $id;
     }
 
+    /**
+     * Removes all DataTree objects owned by a certain user.
+     *
+     * @param string $user  A user name.
+     */
+    function removeUserData($user)
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Move an object to a new parent.
      *
@@ -448,26 +474,38 @@ class DataTree {
     /**
      * Export a list of objects.
      *
-     * @param constant $format   Format of the export
-     * @param string $startleaf  The name of the leaf from which we start the
-     *                           export tree.
-     * @param boolean $reload    Re-load the requested chunk? Defaults to
-     *                           false (only what is currently loaded).
-     * @param string $rootname   The label to use for the root element.
-     *                           Defaults to DATATREE_ROOT.
-     * @param integer $maxdepth  The maximum number of levels to return.
-     *                           Defaults to DATATREE_ROOT, which is no limit.
-     * @param boolean $loadTree  Load a tree starting at $root, or just the
-     *                           requested level and direct parents?
-     *                           Defaults to single level.
+     * @param constant $format      Format of the export
+     * @param string   $startleaf   The name of the leaf from which we start
+     *                              the export tree.
+     * @param boolean  $reload      Re-load the requested chunk? Defaults to
+     *                              false (only what is currently loaded).
+     * @param string   $rootname    The label to use for the root element.
+     *                              Defaults to DATATREE_ROOT.
+     * @param integer  $maxdepth    The maximum number of levels to return.
+     *                              Defaults to DATATREE_ROOT, which is no
+     *                              limit.
+     * @param boolean  $loadTree    Load a tree starting at $root, or just the
+     *                              requested level and direct parents?
+     *                              Defaults to single level.
+     * @param string   $sortby_name Attribute name to use for sorting.
+     * @param string   $sortby_key  Attribute key to use for sorting.
+     * @param integer  $direction   Sort direction:
+     *                               0 - ascending
+     *                               1 - descending
      *
      * @return mixed  The tree representation of the objects, or a PEAR_Error
      *                on failure.
      */
     function get($format, $startleaf = DATATREE_ROOT, $reload = false,
-                 $rootname = DATATREE_ROOT, $maxdepth = -1, $loadTree = false)
+                 $rootname = DATATREE_ROOT, $maxdepth = -1, $loadTree = false,
+                 $sortby_name = null, $sortby_key = null, $direction = 0)
     {
-        $this->_load($startleaf, $loadTree, $reload);
+        /* Set sorting hash */
+        if (!is_null($sortby_name) || !is_null($sortby_name)) {
+            $this->_sortHash = DataTree::sortHash($startleaf, $sortby_name, $sortby_key, $direction);
+        }
+
+        $this->_load($startleaf, $loadTree, $reload, $sortby_name, $sortby_key, $direction);
         $out = array();
 
         switch ($format) {
@@ -494,6 +532,14 @@ class DataTree {
             return PEAR::raiseError('Not supported');
         }
 
+        if (!is_null($this->_sortHash)) {
+            /* Reset sorting hash. */
+            $this->_sortHash = null;
+            /* Reverse since the attribute sorting combined with tree up-ward
+             * sorting produces a reversed object order. */
+            $out = array_reverse($out, true);
+        }
+
         return $out;
     }
 
@@ -510,6 +556,27 @@ class DataTree {
         return $this->_count($startleaf);
     }
 
+    /**
+     * Create attribute sort hash
+     *
+     * @since Horde 3.1
+     *
+     * @param string  $root         The name of the leaf from which we start
+     *                              the export tree.
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                              0 - ascending
+     *                              1 - descending
+     *
+     * @return string The sort hash
+     */
+    function sortHash($root, $sortby_name = null, $sortby_key = null,
+                      $direction = 0)
+    {
+        return sprintf('%s-%s-%s-%s', $root, $sortby_name, $sortby_key, $direction);
+    }
+
     /**
      * Export a list of objects just like get() above, but uses an
      * object id to fetch the list of objects.
@@ -522,7 +589,7 @@ class DataTree {
      * @param string  $rootname   The label to use for the root element.
      *                            Defaults to DATATREE_ROOT.
      * @param integer $maxdepth   The maximum number of levels to return
-     *                            Defaults to DATATREE_ROOT, which is no limit.
+     *                            Defaults to -1, which is no limit.
      *
      * @return mixed  The tree representation of the objects, or a PEAR_Error
      *                on failure.
@@ -583,20 +650,23 @@ class DataTree {
      */
     function set($data, $charset = null)
     {
-        $cats = array();
         $cids = array();
-        foreach ($data as $cat) {
+        foreach ($data as $id => $cat) {
             if (!is_null($charset)) {
                 $cat[1] = String::convertCharset($cat[1], $charset);
             }
             $cids[$cat[0]] = $cat[1];
             $cparents[$cat[0]] = $cat[2];
             $corders[$cat[0]] = $cat[3];
+            $sorders[$cat[0]] = $id;
         }
 
         foreach ($cids as $id => $name) {
             $this->_data[$id]['name'] = $name;
             $this->_data[$id]['order'] = $corders[$id];
+            if (!is_null($this->_sortHash)) {
+                $this->_data[$id]['sorter'][$this->_sortHash] = $sorders[$id];
+            }
             if (!empty($cparents[$id])) {
                 $parents = explode(':', substr($cparents[$id], 1));
                 $par = $parents[count($parents) - 1];
@@ -644,7 +714,12 @@ class DataTree {
             }
         }
 
-        uasort($out, array($this, '_cmp'));
+        if (is_null($this->_sortHash)) {
+            uasort($out, array($this, '_cmp'));
+        } else {
+            uasort($out, array($this, '_cmpSorted'));
+        }
+
         return $out;
     }
 
@@ -1006,6 +1081,30 @@ class DataTree {
         return $name;
     }
 
+    /**
+     * Returns a tree sorted by the specified attribute name and/or key.
+     *
+     * @abstract
+     *
+     * @since Horde 3.1
+     *
+     * @param string $root         Which portion of the tree to sort.
+     *                             Defaults to all of it.
+     * @param boolean $loadTree    Sort the tree starting at $root, or just the
+     *                             requested level and direct parents?
+     *                             Defaults to single level.
+     * @param string $sortby_name  Attribute name to use for sorting.
+     * @param string $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction   Sort direction:
+     *                             0 - ascending
+     *                             1 - descending
+     */
+    function getSortedTree($root, $loadTree = false, $sortby_name = null,
+                           $sortby_key = null, $direction = 0)
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Adds an object.
      *
@@ -1066,6 +1165,8 @@ class DataTree {
     /**
      * Returns the number of objects matching a set of attribute criteria.
      *
+     * @abstract
+     *
      * @see buildAttributeQuery()
      *
      * @param array   $criteria   The array of criteria.
@@ -1084,6 +1185,8 @@ class DataTree {
     /**
      * Returns a set of object ids based on a set of attribute criteria.
      *
+     * @abstract
+     *
      * @see buildAttributeQuery()
      *
      * @param array   $criteria     The array of criteria.
@@ -1108,6 +1211,27 @@ class DataTree {
         return PEAR::raiseError('not supported');
     }
 
+    /**
+     * Sorts IDs by attribute values. IDs without attributes will be added to
+     * the end of the sorted list.
+     *
+     * @abstract
+     *
+     * @param array $unordered_ids  Array of ids to sort.
+     * @param array $sortby_name    Attribute name to use for sorting.
+     * @param array $sortby_key     Attribute key to use for sorting.
+     * @param array $direction      Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
+     *
+     * @return array  Sorted ids.
+     */
+    function sortByAttributes($unordered_ids, $sortby_name = null,
+                              $sortby_key = null, $direction = 0)
+    {
+        return PEAR::raiseError('not supported');
+    }
+
     /**
      * Update the data in an object. Does not change the object's
      * parent or name, just serialized data or attributes.
@@ -1146,6 +1270,30 @@ class DataTree {
         }
     }
 
+     /**
+     * Sorts two objects by their sorter hash field.
+     *
+     * You never call this function; it's used in uasort() calls. Do NOT use
+     * usort(); you'll lose key => value associations.
+     *
+     * @since Horde 3.1
+     *
+     * @param array $a  The first object
+     * @param array $b  The second object
+     *
+     * @return integer  1 if $a should be first,
+     *                 -1 if $b should be first,
+     *                  0 if they are entirely equal.
+     */
+    function _cmpSorted($a, $b)
+    {
+        if ($a['sorter'][$this->_sortHash] < $b['sorter'][$this->_sortHash]) {
+            return 1;
+        } else {
+            return 0;
+        }
+    }
+
     /**
      * Attempts to return a concrete DataTree instance based on $driver.
      *
@@ -1177,10 +1325,12 @@ class DataTree {
         include_once 'Horde/DataTree/' . $driver . '.php';
         $class = 'DataTree_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $dt = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $dt = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+ 
+        return $dt;
     }
 
     /**
