@@ -3,15 +3,14 @@
  * The Horde_Form_Renderer class provides HTML and other renderings of
  * forms for the Horde_Form:: package.
  *
- * $Horde: framework/Form/Form/Renderer.php,v 1.185.4.3 2005/02/05 17:51:34 jan Exp $
+ * $Horde: framework/Form/Form/Renderer.php,v 1.185.4.16 2007/01/02 13:54:17 jan Exp $
  *
- * Copyright 2001-2005 Robert E. Coyle <robertecoyle@hotmail.com>
+ * Copyright 2001-2007 Robert E. Coyle <robertecoyle@hotmail.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Robert E. Coyle <robertecoyle@hotmail.com>
- * @version $Revision: 1.185.4.3 $
  * @package Horde_Form
  */
 class Horde_Form_Renderer {
@@ -19,18 +18,20 @@ class Horde_Form_Renderer {
     var $_name;
     var $_requiredLegend = false;
     var $_requiredMarker = '*';
+    var $_helpMarker = '?';
     var $_onLoadJS = array();
     var $_showHeader = true;
     var $_cols = 2;
     var $_varRenderer = null;
     var $_firstField = null;
+    var $_stripedRows = true;
 
     /**
      * Does the title of the form contain HTML? If so, you are responsible for
      * doing any needed escaping/sanitization yourself. Otherwise the title
      * will be run through htmlspecialchars() before being output.
      *
-     * @var boolean $_encodeTitle
+     * @var boolean
      */
     var $_encodeTitle = true;
 
@@ -38,7 +39,7 @@ class Horde_Form_Renderer {
      * Width of the attributes column.
      *
      * @access private
-     * @var string $_attrColumnWidth
+     * @var string
      */
     var $_attrColumnWidth = '15%';
 
@@ -49,7 +50,7 @@ class Horde_Form_Renderer {
      *                       Possible keys:<code>
      *                       'varrenderer_driver': specifies the driver
      *                           parameter to Horde_UI_VarRenderer::factory().
-     *                       'encode_title': @see $_encodeTitle
+     *                       'encode_title': @see $_encodeTitle</code>
      */
     function Horde_Form_Renderer($params = array())
     {
@@ -112,7 +113,8 @@ class Horde_Form_Renderer {
     function open($action, $method, $name, $enctype = null)
     {
         $this->_name = $name;
-        echo "<form action=\"$action\" method=\"$method\"" . (empty($name) ? '' : " name=\"$name\"") . (is_null($enctype) ? '' : " enctype=\"$enctype\"") . ">\n";
+        $name = htmlspecialchars($name);
+        echo "<form action=\"$action\" method=\"$method\"" . (empty($name) ? '' : " name=\"$name\" id=\"$name\"") . (is_null($enctype) ? '' : " enctype=\"$enctype\"") . ">\n";
         Util::pformInput();
     }
 
@@ -135,42 +137,47 @@ class Horde_Form_Renderer {
 
         $open_section = $form->getOpenSection();
 
-        /* Print out the tabs for all the sections. */
-        echo '<table cellspacing="0"><tr valign="bottom"><td colspan="' .
-            $this->_cols . '">';
-
         /* Add the javascript for the toggling the sections. */
-        Horde::addScriptFile('form_sections.js', 'horde');
-        echo '<script language="JavaScript" type="text/javascript">' . "\n";
-        echo sprintf('var sections_%1$s = new Horde_Form_Sections(\'%1$s\', \'%2$s\');',
-                     $form->getName(),
-                     $open_section);
+        Horde::addScriptFile('form_sections.js', 'horde', true);
+        echo '<script type="text/javascript">' . "\n";
+        printf('var sections_%1$s = new Horde_Form_Sections(\'%1$s\', \'%2$s\');',
+               $form->getName(),
+               $open_section);
         echo "\n" . '</script>';
 
         /* Loop through the sections and print out a tab for each. */
-        echo '<ul class="tabset">';
+        echo "<div class=\"tabset\"><ul>\n";
         foreach ($form->_sections as $section => $val) {
             $class = ($section == $open_section) ? ' class="activeTab"' : '';
             $js = sprintf('onclick="sections_%s.toggle(\'%s\'); return false;"',
                           $form->getName(),
                           $section);
-            printf('<li><a id="_tab_' . $section . '"' . $class . '" href="#" %s>%s%s</a> </li>',
-                   $js,
+            printf('<li%s id="%s"><a href="#" %s>%s%s</a> </li>' . "\n",
+                   $class, '_tab_' . $section, $js,
                    $form->getSectionImage($section),
-                   str_replace(' ', '&nbsp;', $form->getSectionDesc($section)));
+                   $form->getSectionDesc($section));
         }
-        echo '</ul></td></tr></table>';
+        echo "</ul></div><br class=\"clear\" />\n";
     }
 
     function _renderSectionBegin(&$form, $section)
     {
+        // Stripe alternate rows if that option is turned on.
+        if ($this->_stripedRows) {
+            Horde::addScriptFile('stripe.js', 'horde', true);
+            $class = ' class="striped"';
+        } else {
+            $class = '';
+        }
+
         $open_section = $form->getOpenSection();
         if (empty($open_section)) {
             $open_section = '__base';
         }
-        printf('<div id="%s" style="display:%s;"><table class="item" cellspacing="0">',
+        printf('<div id="%s" style="display:%s;"><table%s cellspacing="0">',
                '_section_' . $section,
-               ($open_section == $section ? 'block' : 'none'));
+               ($open_section == $section ? 'block' : 'none'),
+               $class);
     }
 
     function _renderSectionEnd()
@@ -187,10 +194,10 @@ class Horde_Form_Renderer {
     {
         echo "</form>\n";
         if ($focus && !empty($this->_firstField)) {
-            echo '<script language="JavaScript" type="text/javascript">
+            echo '<script type="text/javascript">
 <!--
 try {
-    document.' . $this->_name . '.' . $this->_firstField . '.focus();
+    document.getElementById("' . $this->_firstField . '").focus();
 } catch(e) {}
 //-->
 </script>
@@ -236,8 +243,7 @@ try {
 
         /* Check for a form token error. */
         if (($tokenError = $form->getError('__formToken')) !== null) {
-            printf('<tr class="%s" valign="top"><td colspan="%d"><span class="form-error">%s</span></td></tr>',
-                   $this->getRowClass(),
+            printf('<tr valign="top"><td colspan="%d"><span class="form-error">%s</span></td></tr>',
                    $this->_cols,
                    $tokenError);
         }
@@ -297,8 +303,8 @@ try {
             $this->_renderSectionEnd();
         }
 
-        if (!is_null($error_section)) {
-            echo '<script language="JavaScript" type="text/javascript">' .
+        if (!is_null($error_section) && $form->_sections) {
+            echo '<script type="text/javascript">' .
                 "\n" . sprintf('sections_%s.toggle(\'%s\');',
                                $form->getName(),
                                $error_section) .
@@ -317,7 +323,9 @@ try {
         $this->_renderSubmit($submit, $reset);
     }
 
-    /* Implementation specifics -- begin / end functions. */
+    /**
+     * Implementation specific begin function.
+     */
     function _renderBeginActive($name, $extra)
     {
         echo '<div class="form" id="' . htmlspecialchars($this->_name) . '">';
@@ -329,6 +337,9 @@ try {
         }
     }
 
+    /**
+     * Implementation specific begin function.
+     */
     function _renderBeginInactive($name, $extra)
     {
         echo '<div class="form" id="' . htmlspecialchars($this->_name) . '">';
@@ -337,6 +348,9 @@ try {
         }
     }
 
+    /**
+     * Implementation specific end function.
+     */
     function _renderEnd()
     {
         echo '</div>' . $this->_varRenderer->renderEnd();
@@ -344,7 +358,7 @@ try {
 
     function _renderHeader($header, $error = '')
     {
-?><tr><td class="control" width="100%" colspan="<?php echo $this->_cols; ?>" valign="bottom"><b><?php echo $header ?></b><?php
+?><tr><td class="control" width="100%" colspan="<?php echo $this->_cols ?>" valign="bottom"><strong><?php echo $header ?></strong><?php
         if (!empty($error)) {
 ?><br /><span class="form-error"><?php echo $error ?></span><?php
         }
@@ -354,26 +368,26 @@ try {
 
     function _renderDescription($text)
     {
-?><tr><td width="100%" colspan="<?php echo $this->_cols; ?>" class="<?php echo $this->getRowClass() ?>"><table cellpadding="8"><tr><td><?php echo $text ?></td></tr></table></td></tr>
+?><tr><td width="100%" colspan="<?php echo $this->_cols ?>"><p style="padding:8px"><?php echo $text ?></p></td></tr>
 <?php
     }
 
     function _renderSpacer()
     {
-?><tr><td colspan="<?php echo $this->_cols; ?>">&nbsp;</td></tr>
+?><tr><td colspan="<?php echo $this->_cols ?>">&nbsp;</td></tr>
 <?php
     }
 
     function _renderSubmit($submit, $reset)
     {
-?><table cellspacing="0"><tr><td colspan="<?php echo $this->_cols; ?>" class="control">
+?><div class="control">
   <?php if (!is_array($submit)) $submit = array($submit); foreach ($submit as $submitbutton): ?>
     <input class="button" name="submitbutton" type="submit" value="<?php echo $submitbutton ?>" />
   <?php endforeach; ?>
   <?php if (!empty($reset)): ?>
     <input class="button" name="resetbutton" type="reset" value="<?php echo $reset ?>" />
   <?php endif; ?>
-</td></tr></table>
+</div>
 <?php
     }
 
@@ -382,18 +396,15 @@ try {
     {
         $message = $form->getError($var);
         $isvalid = empty($message);
-        $class = $this->getRowClass();
         echo "<tr valign=\"top\">\n";
-        printf('  <td%s align="right" class="%s">%s%s%s%s</td>' . "\n",
+        printf('  <td%s align="right">%s%s%s%s</td>' . "\n",
                empty($this->_attrColumnWidth) ? '' : ' width="' . $this->_attrColumnWidth . '"',
-               $class,
                $isvalid ? '' : '<span class="form-error">',
                $var->isRequired() ? '<span class="form-error">' . $this->_requiredMarker . '</span>&nbsp;' : '',
                $var->getHumanName(),
                $isvalid ? '' : '<br />' . $message . '</span>');
-        printf('  <td%s class="%s">',
-               ((!$var->hasHelp() && $form->hasHelp()) ? ' colspan="2"' : ''),
-               $class);
+        printf('  <td%s>',
+               ((!$var->hasHelp() && $form->hasHelp()) ? ' colspan="2"' : ''));
     }
 
     function _renderVarInputEnd(&$form, &$var, &$vars)
@@ -405,8 +416,13 @@ try {
 
         /* Display any help for the field. */
         if ($var->hasHelp()) {
-            $class = $this->getRowClass(false);
-            echo "</td>\n  <td align=\"right\" class=\"" . $class . '">' . Help::link($GLOBALS['registry']->getApp(), $var->getHelp()) . '&nbsp';
+            global $registry;
+            if (isset($registry) && is_a($registry, 'Registry')) {
+                $link = Help::link($GLOBALS['registry']->getApp(), $var->getHelp());
+            } else {
+                $link = '<a href="#" onclick="alert(\'' . addslashes(@htmlspecialchars($var->getHelp())) . '\');return false;">' . $this->_helpMarker . '</a>';
+            }
+            echo "</td>\n  <td style=\"text-align:right\">$link&nbsp;";
         }
 
         echo "</td>\n</tr>\n";
@@ -417,15 +433,13 @@ try {
     {
         $message = $form->getError($var);
         $isvalid = empty($message);
-        $class = $this->getRowClass();
         echo "<tr valign=\"top\">\n";
-        printf('  <td%s align="right" class="%s">%s<b>%s</b>%s</td>' . "\n",
+        printf('  <td%s align="right">%s<strong>%s</strong>%s</td>' . "\n",
                empty($this->_attrColumnWidth) ? '' : ' width="' . $this->_attrColumnWidth . '"',
-               $class,
                $isvalid ? '' : '<span class="form-error">',
                $var->getHumanName(),
                $isvalid ? '' : '<br />' . $message . '</span>');
-        printf('  <td class="%s">', $class);
+        echo '  <td>';
     }
 
     function _renderVarDisplayEnd(&$form, &$var, &$vars)
@@ -433,17 +447,6 @@ try {
         echo "</td>\n</tr>\n";
     }
 
-    function getRowClass($increment = true)
-    {
-        static $i = 1;
-
-        if (!$increment) {
-            return 'item' . ($i % 2);
-        }
-
-        return 'item' . (++$i % 2);
-    }
-
     function _sectionHeader($title, $extra = '')
     {
         if (strlen($title)) {
@@ -482,7 +485,8 @@ try {
          * specified. */
         $renderer = basename($renderer);
         if (empty($renderer) || (strcmp($renderer, 'none') == 0)) {
-            return $ret = &new Horde_Form_Renderer($params);
+            $formrenderer = &new Horde_Form_Renderer($params);
+            return $formrenderer;
         }
 
         if (!empty($app)) {
@@ -494,10 +498,12 @@ try {
         }
         $class = 'Horde_Form_Renderer_' . $renderer;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $formrenderer = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $formrenderer = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $formrenderer;
     }
 
     /**
