@@ -2,11 +2,11 @@
 /**
  * This is a utility class, every method is static.
  *
- * $Horde: framework/LDAP/LDAP.php,v 1.5.12.10 2006/01/01 21:28:23 jan Exp $
+ * $Horde: framework/LDAP/LDAP.php,v 1.5.12.12 2007/07/19 21:04:56 chuck Exp $
  *
- * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
@@ -20,19 +20,23 @@ class Horde_LDAP {
      *
      * @static
      *
-     * @param string $lhs  The attribute to test.
-     * @param string $op   The operator.
-     * @param string $rhs  The comparison value.
+     * @param string $lhs    The attribute to test.
+     * @param string $op     The operator.
+     * @param string $rhs    The comparison value.
+     * @param array $params  Any additional parameters for the operator. @since
+     *                       Horde 3.2
      *
      * @return string  The LDAP search fragment.
      */
-    function buildClause($lhs, $op, $rhs)
+    function buildClause($lhs, $op, $rhs, $params = array())
     {
         switch ($op) {
         case 'LIKE':
-            return empty($rhs) ?
-                sprintf('(%s=*)', $lhs) :
-                sprintf('(%s=*%s*)', $lhs, Horde_LDAP::quote($rhs));
+            return empty($rhs)
+                ? sprintf('(%s=*)', $lhs)
+                : (empty($params['begin'])
+                   ? sprintf('(%s=*%s*)', $lhs, Horde_LDAP::quote($rhs))
+                   : sprintf('(|(%s=%s*)(%s=* %s*))', $lhs, Horde_LDAP::quote($rhs), $lhs, Horde_LDAP::quote($rhs)));
 
         default:
             return sprintf('(%s%s%s)', $lhs, $op, Horde_LDAP::quote($rhs));
