@@ -2,7 +2,7 @@
 /**
  * PostgreSQL Session Handler for PHP (native).
  *
- * Copyright 2000-2005 Jon Parise <jon@csh.rit.edu>.  All rights reserved.
+ * Copyright 2000-2007 Jon Parise <jon@csh.rit.edu>.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -49,7 +49,7 @@
  *  pat@pcprogrammer.com    Perform update in a single transaction
  *  Jonathan Crompton       Lock row for life of session</pre>
  *
- * $Horde: framework/SessionHandler/SessionHandler/pgsql.php,v 1.12.10.12 2005/10/15 20:35:01 chuck Exp $
+ * $Horde: framework/SessionHandler/SessionHandler/pgsql.php,v 1.12.10.16 2007/01/02 13:54:38 jan Exp $
  *
  * @author  Jon Parise <jon@csh.rit.edu>
  * @since   Horde 3.0
@@ -169,6 +169,10 @@ class SessionHandler_pgsql extends SessionHandler {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         $result = @pg_query($this->_db, $query);
+        if (!$result) {
+            Horde::logMessage('Error writing session data: ' . pg_last_error($this->_db), __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
         $rows = pg_affected_rows($result);
         pg_free_result($result);
 
